@@ -12,19 +12,22 @@ class MemcachedStorage(Storage):
     """
     Rate limit storage with memcached as backend.
 
-    Depends on the `pymemcache` library.
+    Depends on :pypi:`pymemcache`.
     """
 
-    MAX_CAS_RETRIES = 10
     STORAGE_SCHEME = ["memcached"]
+    """The storage scheme for memcached"""
 
     def __init__(self, uri: str, **options):
         """
         :param uri: memcached location of the form
-         `memcached://host:port,host:port`, `memcached:///var/tmp/path/to/sock`
+         ``memcached://host:port,host:port``,
+         ``memcached:///var/tmp/path/to/sock``
         :param options: all remaining keyword arguments are passed
-         directly to the constructor of :class:`pymemcache.client.base.Client`
-        :raise ConfigurationError: when `pymemcache` is not available
+         directly to the constructor of :class:`pymemcache.client.base.PooledClient`
+         or :class:`pymemcache.client.hash.HashClient` (if there are more than
+         one hosts specified)
+        :raise ConfigurationError: when :pypi:`pymemcache` is not available
         """
         parsed = urllib.parse.urlparse(uri)
         self.hosts = []
@@ -39,7 +42,7 @@ class MemcachedStorage(Storage):
                 self.hosts = [parsed.path]  # type: ignore
 
         self.library = options.pop("library", "pymemcache.client")
-        self.cluster_library = options.pop("library", "pymemcache.client.hash")
+        self.cluster_library = options.pop("cluster_library", "pymemcache.client.hash")
         self.client_getter = options.pop("client_getter", self.get_client)
         self.options = options
 
@@ -54,20 +57,20 @@ class MemcachedStorage(Storage):
     def get_client(self, module, hosts, **kwargs):
         """
         returns a memcached client.
+
         :param module: the memcached module
         :param hosts: list of memcached hosts
-        :return:
         """
         return (
             module.HashClient(hosts, **kwargs)
             if len(hosts) > 1
-            else module.Client(*hosts, **kwargs)
+            else module.PooledClient(*hosts, **kwargs)
         )
 
     def call_memcached_func(self, func, *args, **kwargs):
         if "noreply" in kwargs:
-            argspec = inspect.getargspec(func)
-            if not ("noreply" in argspec.args or argspec.keywords):
+            argspec = inspect.getfullargspec(func)
+            if not ("noreply" in argspec.args or argspec.varkw):
                 kwargs.pop("noreply")  # noqa
         return func(*args, **kwargs)
 
@@ -111,17 +114,9 @@ class MemcachedStorage(Storage):
         if not self.call_memcached_func(
             self.storage.add, key, 1, expiry, noreply=False
         ):
+            value = self.storage.incr(key, 1) or 1
             if elastic_expiry:
-                value, cas = self.storage.gets(key)
-                retry = 0
-                while (
-                    not self.call_memcached_func(
-                        self.storage.cas, key, int(value or 0) + 1, cas, expiry
-                    )
-                    and retry < self.MAX_CAS_RETRIES
-                ):
-                    value, cas = self.storage.gets(key)
-                    retry += 1
+                self.call_memcached_func(self.storage.touch, key, expiry)
                 self.call_memcached_func(
                     self.storage.set,
                     key + "/expires",
@@ -129,16 +124,15 @@ class MemcachedStorage(Storage):
                     expire=expiry,
                     noreply=False,
                 )
-                return int(value or 0) + 1
-            else:
-                return self.storage.incr(key, 1) or 1
-        self.call_memcached_func(
-            self.storage.set,
-            key + "/expires",
-            expiry + time.time(),
-            expire=expiry,
-            noreply=False,
-        )
+            return value
+        else:
+            self.call_memcached_func(
+                self.storage.set,
+                key + "/expires",
+                expiry + time.time(),
+                expire=expiry,
+                noreply=False,
+            )
         return 1
 
     def get_expiry(self, key: str) -> int:
@@ -149,7 +143,8 @@ class MemcachedStorage(Storage):
 
     def check(self) -> bool:
         """
-        check if storage is healthy
+        Check if storage is healthy by calling the ``get`` command
+        on the key ``limiter-check``
         """
         try:
             self.call_memcached_func(self.storage.get, "limiter-check")
