@@ -1,80 +1,24 @@
 import time
-from typing import Any
+from typing import Any, Tuple
 
-from ..util import get_dependency
-from ..errors import ConfigurationError
-from .base import Storage
+from ..util import get_package_data
+from .base import MovingWindowSupport, Storage
 
 
 class RedisInteractor(object):
-    SCRIPT_MOVING_WINDOW = """
-        local items = redis.call('lrange', KEYS[1], 0, tonumber(ARGV[2]))
-        local expiry = tonumber(ARGV[1])
-        local a = 0
-        local oldest = nil
+    RES_DIR = "resources/redis/lua_scripts"
 
-        for idx=1,#items do
-            if tonumber(items[idx]) >= expiry then
-                a = a + 1
+    SCRIPT_MOVING_WINDOW = get_package_data(f"{RES_DIR}/moving_window.lua")
+    SCRIPT_ACQUIRE_MOVING_WINDOW = get_package_data(
+        f"{RES_DIR}/acquire_moving_window.lua"
+    )
+    SCRIPT_CLEAR_KEYS = get_package_data(f"{RES_DIR}/clear_keys.lua")
+    SCRIPT_INCR_EXPIRE = get_package_data(f"{RES_DIR}/incr_expire.lua")
 
-                if oldest == nil then
-                    oldest = tonumber(items[idx])
-                end
-            else
-                break
-            end
-        end
-
-        return {oldest, a}
-        """
-
-    SCRIPT_ACQUIRE_MOVING_WINDOW = """
-        local entry = redis.call('lindex', KEYS[1], tonumber(ARGV[2]) - 1)
-        local timestamp = tonumber(ARGV[1])
-        local expiry = tonumber(ARGV[3])
-
-        if entry and tonumber(entry) >= timestamp - expiry then
-            return false
-        end
-        local limit = tonumber(ARGV[2])
-        local no_add = tonumber(ARGV[4])
-
-        if 0 == no_add then
-            redis.call('lpush', KEYS[1], timestamp)
-            redis.call('ltrim', KEYS[1], 0, limit - 1)
-            redis.call('expire', KEYS[1], expiry)
-        end
-
-        return true
-        """
-
-    SCRIPT_CLEAR_KEYS = """
-        local keys = redis.call('keys', KEYS[1])
-        local res = 0
-
-        for i=1,#keys,5000 do
-            res = res + redis.call(
-                'del', unpack(keys, i, math.min(i+4999, #keys))
-            )
-        end
-
-        return res
-        """
-
-    SCRIPT_INCR_EXPIRE = """
-        local current
-        current = redis.call("incr",KEYS[1])
-
-        if tonumber(current) == 1 then
-            redis.call("expire",KEYS[1],ARGV[1])
-        end
-
-        return current
-    """
     lua_moving_window: Any
     lua_acquire_window: Any
 
-    def get_moving_window(self, key: str, limit: int, expiry):
+    def get_moving_window(self, key, limit, expiry) -> Tuple[int, int]:
         """
         returns the starting point and the number of entries in the moving
         window
@@ -85,9 +29,10 @@ class RedisInteractor(object):
         """
         timestamp = time.time()
         window = self.lua_moving_window([key], [int(timestamp - expiry), limit])
-        return window or (timestamp, 0)
 
-    def _incr(self, key: str, expiry: int, connection, elastic_expiry=False):
+        return window or (int(timestamp), 0)
+
+    def _incr(self, key: str, expiry: int, connection, elastic_expiry=False) -> int:
         """
         increments the counter for a given rate limit key
 
@@ -96,15 +41,18 @@ class RedisInteractor(object):
         :param expiry: amount in seconds for the key to expire in
         """
         value = connection.incr(key)
+
         if elastic_expiry or value == 1:
             connection.expire(key, expiry)
+
         return value
 
-    def _get(self, key: str, connection):
+    def _get(self, key: str, connection) -> int:
         """
         :param connection: Redis connection
         :param key: the key to get the counter value for
         """
+
         return int(connection.get(key) or 0)
 
     def _clear(self, key: str, connection):
@@ -114,22 +62,16 @@ class RedisInteractor(object):
         """
         connection.delete(key)
 
-    def _acquire_entry(
-        self, key: str, limit: int, expiry: int, connection, no_add=False
-    ) -> bool:
+    def _acquire_entry(self, key: str, limit: int, expiry: int, connection) -> bool:
         """
         :param key: rate limit key to acquire an entry in
         :param limit: amount of entries allowed
         :param expiry: expiry of the entry
-        :param no_add: if False an entry is not actually acquired but
-         instead serves as a 'check'
         :param connection: Redis connection
-        :return: True/False
         """
         timestamp = time.time()
-        acquired = self.lua_acquire_window(
-            [key], [timestamp, limit, expiry, int(no_add)]
-        )
+        acquired = self.lua_acquire_window([key], [timestamp, limit, expiry])
+
         return bool(acquired)
 
     def _get_expiry(self, key: str, connection=None) -> int:
@@ -137,6 +79,7 @@ class RedisInteractor(object):
         :param key: the key to get the expiry for
         :param connection: Redis connection
         """
+
         return int(max(connection.ttl(key), 0) + time.time())
 
     def _check(self, connection) -> bool:
@@ -150,36 +93,35 @@ class RedisInteractor(object):
             return False
 
 
-class RedisStorage(RedisInteractor, Storage):
+class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
     """
     Rate limit storage with redis as backend.
 
-    Depends on the `redis-py` library.
+    Depends on :pypi:`redis`.
     """
 
     STORAGE_SCHEME = ["redis", "rediss", "redis+unix"]
+    """The storage scheme for redis"""
+
+    DEPENDENCIES = ["redis"]
 
     def __init__(self, uri: str, **options):
         """
-        :param uri: uri of the form `redis://[:password]@host:port`,
-         `redis://[:password]@host:port/db`,
-         `rediss://[:password]@host:port`, `redis+unix:///path/to/sock` etc.
+        :param uri: uri of the form ``redis://[:password]@host:port``,
+         ``redis://[:password]@host:port/db``,
+         ``rediss://[:password]@host:port``, ``redis+unix:///path/to/sock`` etc.
          This uri is passed directly to :func:`redis.from_url` except for the
-         case of `redis+unix` where it is replaced with `unix`.
+         case of ``redis+unix://`` where it is replaced with ``unix://``.
         :param options: all remaining keyword arguments are passed
          directly to the constructor of :class:`redis.Redis`
-        :raise ConfigurationError: when the redis library is not available
+        :raise ConfigurationError: when the :pypi:`redis` library is not available
         """
-        redis = get_dependency("redis")
-        if not redis:
-            raise ConfigurationError(
-                "redis prerequisite not available"
-            )  # pragma: no cover
+        super().__init__()
+        redis = self.dependencies["redis"]
         uri = uri.replace("redis+unix", "unix")
 
         self.storage = redis.from_url(uri, **options)
         self.initialize_storage(uri)
-        super(RedisStorage, self).__init__()
 
     def initialize_storage(self, _uri: str):
         self.lua_moving_window = self.storage.register_script(self.SCRIPT_MOVING_WINDOW)
@@ -198,48 +140,48 @@ class RedisStorage(RedisInteractor, Storage):
         :param key: the key to increment
         :param expiry: amount in seconds for the key to expire in
         """
+
         if elastic_expiry:
-            return super(RedisStorage, self)._incr(
-                key, expiry, self.storage, elastic_expiry
-            )
+            return super()._incr(key, expiry, self.storage, elastic_expiry)
         else:
             return self.lua_incr_expire([key], [expiry])
 
-    def get(self, key: str):
+    def get(self, key: str) -> int:
         """
         :param key: the key to get the counter value for
         """
-        return super(RedisStorage, self)._get(key, self.storage)
+
+        return super()._get(key, self.storage)
 
     def clear(self, key: str):
         """
         :param key: the key to clear rate limits for
         """
-        return super(RedisStorage, self)._clear(key, self.storage)
 
-    def acquire_entry(self, key: str, limit: int, expiry: int, no_add=False):
+        return super()._clear(key, self.storage)
+
+    def acquire_entry(self, key: str, limit: int, expiry: int):
         """
         :param key: rate limit key to acquire an entry in
         :param limit: amount of entries allowed
         :param expiry: expiry of the entry
-        :param no_add: if False an entry is not actually acquired but
-         instead serves as a 'check'
         """
-        return super(RedisStorage, self)._acquire_entry(
-            key, limit, expiry, self.storage, no_add=no_add
-        )
+
+        return super()._acquire_entry(key, limit, expiry, self.storage)
 
     def get_expiry(self, key: str) -> int:
         """
         :param key: the key to get the expiry for
         """
-        return super(RedisStorage, self)._get_expiry(key, self.storage)
+
+        return super()._get_expiry(key, self.storage)
 
     def check(self) -> bool:
         """
         check if storage is healthy
         """
-        return super(RedisStorage, self)._check(self.storage)
+
+        return super()._check(self.storage)
 
     def reset(self) -> int:
         """
@@ -254,4 +196,5 @@ class RedisStorage(RedisInteractor, Storage):
         """
 
         cleared = self.lua_clear_keys(["LIMITER*"])
+
         return cleared
